|FPRIMS32.S - rsalib assembler primitives for MC680x0
|             (Pure-C/Atari ST version, 32-bit units)
|
|Written by Stephan Baucke 19-Oct-91
|Assembler: Pure-PASM
|
|On systems with 68020 or higher it might be faster to use 32-Bit units
|instead of 16 Bits (I didn't yet test this, but there is no improvement
|on a plain 68000).
|
|Note that the function P_SETP of the Intel primitives is not needed here.
|`set_precision' has to be defined just like in `PORTABLE' mode in rsalib.h

	IMPORT global_precision
	P_SMUL

	.text

|boolean P_ADDC(unitptr r1, unitptr r2, boolean carry);
| /* multiprecision add with carry r2 to r1, result in r1 */
|Parameters: A0.l: r1, A1.l: r2, D0.b: carry
|Result:     D0.b: new carry
|Modifies: D0-D2/A0-A1

P_ADDC:	moveb  d0,d1       |boolean carry
	sne     d1          |set bit 0 to carry
	movew  global_precision,d2 |# of units
	movew  d2,d0       |make copy
	lslw   #2,d2       |calc # of bytes (4 per unit)
	addw   d2,a0       |point r1 to least significant unit
	addw   d2,a1       |point r2 to least significant unit
	lsrw   #1,d2       |divide back (now 2 * units)
	andw   #2*7,d2     |yields 2 * (units % 8)
	negw   d2          |negative offset (for 2-Byte instructions)
	lsrw   #3,d0       |units / 8
	beq   .onebyone   |skip loop if zero count

	subqw  #1,d0       |one off (dbf counter)
	lsrb   d1          |set X if carry
.loop:	REPT    8               |8 units per run
	addxl a1@-,a0@-
	ENDM
	dbf     d0,.loop
	jmp     pc@(.base,d2:w)  |do remaining units

.onebyone:	lsrb   d1              |set X if carry
	jmp     pc@(.base,d2:w)  |do remaining units

	REPT    7               |max 7 units remaining
	addxl a1@-,a0@-    |(2-byte instruction)
	ENDM
.base:	scs     d0              |set returned carry
	rts

|boolean P_SUBB(unitptr r1, unitptr r2, boolean borrow);
| /* multiprecision subtract with borrow, r2 from r1, result in r1 */
|Parameters: A0.l: r1, A1.l: r2, D0.b: borrow
|Result:     D0.b: new borrow
|Modifies: D0-D2/A0/A1

P_SUBB:	moveb  d0,d1       |boolean carry
	sne     d1          |set bit 0 to carry
	movew  global_precision,d2 |# of units
	movew  d2,d0       |make copy
	lslw   #2,d2       |calc # of bytes (4 per unit)
	addw   d2,a0       |point r1 to least significant unit
	addw   d2,a1       |point r2 to least significant unit
	lsrw   #1,d2       |divide back (now 2 * units)
	andw   #2*7,d2     |yields 2 * (units % 8)
	negw   d2          |negative offset (for 2-byte instructions)
	lsrw   #3,d0       |units / 8
	beq   .onebyone   |skip loop if zero count

	subqw  #1,d0       |one off (dbf counter)
	lsrb   d1          |set X if carry
.loop:	REPT    8               |8 units per run
	subxl a1@-,a0@-
	ENDM
	dbf     d0,.loop
	jmp     pc@(.base,d2:w)  |do remaining units

.onebyone:	lsrb   d1              |set X if carry
	jmp     pc@(.base,d2:w)  |do remaining units

	REPT    7               |max 7 units remaining
	subxl a1@-,a0@-    |(2-byte instruction)
	ENDM
.base:	scs     d0              |set returned carry
	rts

|boolean P_ROTL(unitptr r1, boolean carry);
| /* multiprecision rotate left 1 bit with carry, result in r1. */
|Parameters: A0.l: r1, D0.b: carry
|Result:     D0.b: new carry
|Modifies: D0-D2/A0

P_ROTL:	moveb  d0,d1       |boolean carry
	sne     d1          |set bit 0 to carry
	movew  global_precision,d2 |# of units
	movew  d2,d0       |make copy
	lslw   #2,d2       |calc # of bytes (4 per unit)
	addw   d2,a0       |point r1 to least significant unit
	andw   #4*7,d2     |yields 4 * (units % 8)
	negw   d2          |negative offset (for 4-byte instructions)
	lsrw   #3,d0       |units / 8
	beq   .onebyone   |skip loop if zero count

	subqw  #1,d0       |one off (dbf counter)
	lsrb   d1          |set X if carry
.loop:	REPT    8           |8 units per run
	roxlw a0@-      |(roxl.l <ea> is not allowed on the 68000)
	roxlw a0@-
	ENDM
	dbf     d0,.loop
	jmp     pc@(.base,d2:w)  |do remaining units

.onebyone:	lsrb   d1              |set X if carry
	jmp     pc@(.base,d2:w)  |do remaining units

	REPT    7               |max 7 units remaining
	roxlw a0@-          |(4 bytes instructions per unit)
	roxlw a0@-
	ENDM
.base:	scs     d0              |set returned carry
	rts

            
|void P_SETP(short nbits);
| /* sets working precision to specified number of bits. */
| /* only to minimize portation differences              */
|Parameters: --
|Result:     --
         
P_SETP:	rts

| ***NOT TESTED***
|
|void P_SMUL(MULTUNIT *prod, MULTUNIT *multiplicand, MULTUNIT multiplier)
| /* multiprecision multiply */
|Parameters: A0.l: prod, A1.l: multiplicand, D0.w: multiplier
|Modifies: D0-D3/A0-A1
|Result:     --

P_SMUL:	movew  global_precision,d3
	lslw   #1,d3
	subqw  #1,d3
	clrl   d2
	clrl   d1

.loop:	movew  a1@-,d1
	mulul  d0,d1
	addl   d2,d1
	addw   d1,a0@-
	scs     d2
	lsrl   #16,d1
	addw   d1,d2
	dbf     d3,.loop

	movew  d2,a0@
	rts


	| END

